//
//***************************************************************************
// Name    : USBDescriptor.bas                                              *
// Author  : David John Barker                                              *
// Notice  : Copyright (c) 2007 Mecanique                                   *
//         : All Rights Reserved                                            *
// Date    : 10/01/2007                                                     *
// Version : 1.0                                                            *
// Notes   : Default USB HID descriptor module                              *
//         : =================== DO not EDIT THIS FILE ===================  *
//         : save a copy IN your current USB project folder and edit. if    *
//         : you rename the NEW descriptor file, then don't forget to set   *
//         : the new filename at the top of your main program using...      *
//         : #option USB_DESCRIPTOR = "MyDescriptor.bas"                    *
//***************************************************************************
//
module CDCDescriptor

// import modules...
include "usbconfig.bas"
include "usbdefs.bas"

// Device Descriptor
public const device_dsc(USB_DEV_DSC_SIZE) as byte = (                                         'usbdsc.c
    sizeof(USB_DEV_DSC),            // bLength - size of this descriptor IN bytes
    DSC_DEV,                        // bDescriptorType - device descriptor type
    $00,                            // bcdUSB (low byte)  - USB Spec Release Number IN BCD format (2.0)
    $02,                            // bcdUSB (high byte) - USB Spec Release Number IN BCD format (2.0)
    CDC_DEVICE,                     // bDeviceClass - class Code
    $00,                            // bDeviceSubClass - subclass code
    $00,                            // bDeviceProtocol - protocol code
    USB_EP0_BUFF_SIZE,              // bMaxPacketSize - max packet size for EP0
    USB_VID and $00FF,              // idVendor (low byte)
    USB_VID >> 8,                   // idVendor (high byte)
    USB_PID and $00FF,              // idProduct (low byte)
    USB_PID >> 8,                   // idProduct (high byte)
    USB_DEVICE_VERSION and $00FF,   // bcdDevice (low byte) - device release number IN BCD format
    USB_DEVICE_VERSION >> 8,        // bcdDevice (high byte) - device release number IN BCD format
    USB_MANUFACTURER_INDEX,         // iManufacturer - manufacturer string index
    USB_PRODUCT_INDEX,              // iProduct - product string index
    USB_SERIAL_INDEX,               // iSerialNumber - device serial number string index
    $01                             // bNumConfigurations - mumber of possible configurations
)

// A typical configuration descriptor consists of these components:
//     At least one configuration descriptor (USB_CFG_DSC)
//     One or more interface descriptors (USB_INTF_DSC)
//     One or more endpoint descriptors (USB_EP_DSC)
// In addition, there is usually a descriptor string that provides a
// plain text description of the device

// Configuration 1 Descriptor
const CFG01_SIZE = 67
public const cfg01(CFG01_SIZE) as byte = (                                      'usbdsc.c

    // configuration Descriptor (cd01)
    sizeof(USB_CFG_DSC),    // Size of this descriptor in bytes
    DSC_CFG,                // CONFIGURATION descriptor type
    CFG01_SIZE,             // sizeof(cfg01) - Total length of data for this cfg
    0,                      // HIGH BYTE
    2,                      // Number of interfaces in this cfg
    1,                      // Index value of this configuration
    0,                      // Configuration string index
    _DEFAULT,               // Attributes, see usbdefs_std_dsc.h
    USB_BUS_POWER,          // Max power consumption (2X mA)

    // interface descriptor (i00a00)
    sizeof(USB_INTF_DSC),   // Size of this descriptor in bytes
    DSC_INTF,               // INTERFACE descriptor type
    0,                      // Interface Number
    0,                      // Alternate Setting Number
    1,                      // Number of endpoints in this intf
    COMM_INTF,              // Class code
    ABSTRACT_CONTROL_MODEL, // Subclass code
    V25TER,                 // Protocol code
    0,                      // Interface string index

    // CDC class-specific descriptors
    sizeof(USB_CDC_HEADER_FN_DSC),   // size of descripor in bytes
    CS_INTERFACE,                    // descriptor type
    DSC_FN_HEADER,                   // sub type
    $10,                             // CDC (low byte) BCD
    $01,                             // CDC (high byte) BCD

    sizeof(USB_CDC_ACM_FN_DSC),      // size of descritor in bytes
    CS_INTERFACE,                    // descriptor type
    DSC_FN_ACM,                      // sub type
    $02,                             // capabilities

    sizeof(USB_CDC_UNION_FN_DSC),    // size of descriptor in bytes
    CS_INTERFACE,                    // descriptor type
    DSC_FN_UNION,                    // sub type
    CDC_COMM_INTF_ID,                // master int
    CDC_DATA_INTF_ID,                // slave int

    sizeof(USB_CDC_CALL_MGT_FN_DSC), // size of descriptor in bytes
    CS_INTERFACE,                    // descriptor type
    DSC_FN_CALL_MGT,                 // sub type
    $00,                             // capabilities
    CDC_DATA_INTF_ID,                // data interface

    // Endpoint Descriptor
    sizeof(USB_EP_DSC),              // size of descriptor in bytes
    DSC_EP,                          // descriptor type
    _EP02_IN,                        // EP address
    _INT,                            // attributes
    CDC_INT_EP_SIZE,                 // size (low byte)
    $00,                             // size (high byte)
    $02,                             // interval

    // Interface Descriptor
    sizeof(USB_INTF_DSC),   // Size of this descriptor in bytes
    DSC_INTF,               // INTERFACE descriptor type
    1,                      // Interface Number
    0,                      // Alternate Setting Number
    2,                      // Number of endpoints in this intf
    DATA_INTF,              // Class code
    0,                      // Subclass code
    NO_PROTOCOL,            // Protocol code
    0,                      // Interface string index

    // Endpoint Descriptors
    sizeof(USB_EP_DSC),     // size of descriptor in bytes
    DSC_EP,                 // descriptor type
    _EP03_OUT,              // EP address
    _BULK,                  // attributes
    CDC_BULK_OUT_EP_SIZE,   // max packet size (low byte)
    $00,                    // max packet size (high byte)
    $00,                    // interval

    sizeof(USB_EP_DSC),     // size of descriptor in bytes
    DSC_EP,                 // descriptor type
    _EP03_IN,               // EP address
    _BULK,                  // attributes
    CDC_BULK_IN_EP_SIZE,    // max packet interval (low byte)
    $00,                    // max packet interval (high byte)
    $00                     // interval
)

// language string - unicode format...
const sd000_SIZE = 2     // size of data (2 bytes)
public const sd000(sd000_SIZE+2) as byte = (                                               'usbdsc.c
    sd000_SIZE+2,    // Size of this descriptor in bytes
    DSC_STR,        // DescriptorType
    $09,            // language ID - low byte
    $04             // language ID - high byte
)

// manufacturer string - unicode format...
const sd001_SIZE = 9*2       // length of unicode string data * 2
public const sd001(sd001_SIZE+2) as byte = (                                              'usbdsc.c
    sd001_SIZE+2,    // Size of this descriptor in bytes
    DSC_STR,        // DescriptorType
    "M", 0, "e", 0, "c", 0, "a", 0, "n", 0, "i", 0, "q", 0, "u", 0, "e", 0
)

// product string - unicode format...
const sd002_SIZE = 26*2      // length of unicode string data * 2
public const sd002(sd002_SIZE+2) as byte = (                                              'usbdsc.c
    sd002_SIZE+2,     // Size of this descriptor in bytes
    DSC_STR,         // DescriptorType
    "S", 0, "w", 0, "o", 0, "r", 0, "d", 0, "f", 0, "i", 0, "s", 0, "h", 0, " ", 0,
    "V", 0, "i", 0, "r", 0, "t", 0, "u", 0, "a", 0, "l", 0, " ", 0,
    "C", 0, "O", 0, "M", 0, " ", 0, "P", 0, "o", 0, "r", 0, "t", 0
)

// product serial number...
#if USB_HAVE_SERIAL_STRING
const sd003_SIZE = 4*2       // length of unicode string data * 2
public const sd003(sd003_SIZE+2) as byte = (                                              'usbdsc.c
    sd003_SIZE+2,    // Size of this descriptor in bytes
    DSC_STR,        // DescriptorType
    "1", 0, ".", 0, "0", 0, "0", 0
)
#endif
