//
//***************************************************************************
// Name    : main_cdc.bas                                                   *
// Author  : Jerry Messina                                                  *
// Notice  : Copyright (c) 20011                                            *
//         : All Rights Reserved                                            *
// Date    : 4/7/2011                                                       *
// Version : 1.1                                                            *
// Notes   : USB CDC main application example                               *
//         :                                                                *
// Ver 1.1 : added 'SF startup initialization' section to ensure register   *
//         : settings at poweron when used with a bootloader                *
//***************************************************************************
//

//
//-----------------------------------------------------------------------------
// hardware platforms (uncomment one section/customize for your hardware)
//-----------------------------------------------------------------------------
//
'{
// Sparkfun USB whacker hardware
device = 18F2553
clock = 48
#define USB_WHACKER
'}

{
// Microchip LPC (LowPinCount) 14K50 USB development board
device = 18F14K50
clock = 48
#define LPC_DEVBD
}

{
// 46J50 eval board hardware (USB self-powered)
device = 18F46J50
clock = 48
#define EVAL_J50
}

//
//-----------------------------------------------------------------------------
// config settings
//-----------------------------------------------------------------------------
//
#if defined(USB_WHACKER)
'18F2553
config
    FOSC = HSPLL_HS,         ' External 24MHz xtal
    PLLDIV = 6,
    CPUDIV = OSC1_PLL2,      ' 48MHz cpu clock
    USBDIV = 2,              ' 48MHz usb clock
    FCMEN = OFF,
    IESO = OFF,
    PWRT = ON,
    BOR = OFF,
    BORV = 2,
    VREGEN = ON,
    WDT = OFF,
    WDTPS = 2048,
    CCP2MX = OFF,
    PBADEN = OFF,
    LPT1OSC = OFF,
    MCLRE = ON,
    STVREN = ON,
    LVP = OFF,
    XINST = OFF,
    DEBUG = OFF
#endif  // USB_WHACKER


#if defined(LPC_DEVBD)
'18F14K50
config
    FOSC = HS,              ' External 12MHz xtal
    CPUDIV = NOCLKDIV,      ' 48MHz cpu clock
    USBDIV = OFF,
    PLLEN = ON,             ' PLL enabled
    PCLKEN = ON,
    IESO = OFF,
    FCMEN = OFF,
    PWRTEN = ON,
    BOREN = ON,
    BORV = 30,
    WDTEN = OFF,
    WDTPS = 2048,
    HFOFST = ON,
    MCLRE = ON,            ' must be off to read switch S1
    STVREN = ON,
    LVP = OFF,
    BBSIZ = OFF,
    XINST = OFF,
    DEBUG = OFF
#endif      // LPC_DEVBD


#if defined(EVAL_J50)
'18F46J50
public config
   OSC = INTOSCPLLO,                  ' internal osc 8MHz
   PLLDIV = 2,                        ' usb 48MHz
   CPUDIV = OSC1,                     ' 48MHz cpu clock
   FCMEN = OFF,
   IESO = OFF,
   WDTEN = OFF,
   WDTPS = 2048,
   STVREN = ON,
   XINST = OFF,
   DEBUG = OFF
#endif

//
//-----------------------------------------------------------------------------
// enable display of status messages to the Results window
//-----------------------------------------------------------------------------
//
// set false to disable messages
#option SHOW_BUILD_MSGS = true          // for this module
#option SHOW_USB_BUILD_MSGS = true      // for usb modules (off by default)

//
//-----------------------------------------------------------------------------
// program build options (ORG_RESET and/or ORG_PROGRAM)
//-----------------------------------------------------------------------------
//
// build for bootloader/standalone operation
#variable BUILD_FOR_BOOTLOADER = false
#variable APP_PROGRAM_START = $001000       // must agree with bootloader setting

#if (BUILD_FOR_BOOTLOADER = true)           // build for use with bootloader
  #option org_reset = APP_PROGRAM_START
  #if (SHOW_BUILD_MSGS)
    #warning "building for use with bootloader"
  #endif
#else                                       // build for standalone use (no bootloader)
  #variable APP_PROGRAM_START = $000000     // normal location for startup
  #option org_program = APP_PROGRAM_START   // (can use this to offset location to match bootloader use)
#endif

//
//-----------------------------------------------------------------------------
// SF startup initialization
//-----------------------------------------------------------------------------
// note: include system.bas first before any other files
// this is VERY important if you're using a bootloader (requires system.bas V1.4 or later)
// be sure to set any system.bas WDT options prior to including the file
'#option WDT = true
include "system.bas"
#if not defined(SF_SYSTEM_VERSION) or (SF_SYSTEM_VERSION < 14)
  #if (BUILD_FOR_BOOTLOADER = true)
    #error "bootloader use requires system.bas v1.4 or later"
  #endif
#endif

//
//-------------------------------------------------------------
// USB options and includes
//-------------------------------------------------------------
//
// disable extended ram use
// note: this option will not enable extended ram, only disable it (see usbmem.bas)
'#option USB_DISABLE_EXTENDED_RAM = true

// interrupt/polled mode
// note: you do not have to change the source when switching between modes... calls to
// USBService() will be optimized out when compiling for interrupt mode (USB_SERVICE=true)
// this option is set true by default in USBConfig.bas
#option USB_SERVICE = true

// import modules...
include "usbcdc.bas"

//
//-------------------------------------------------------------
// usb whacker hardware
//-------------------------------------------------------------
//
#if defined(USB_WHACKER)
dim YELLOW_LED  as PORTC.0      // LED2
dim RED_LED     as PORTC.1      // LED3
dim PB_SWITCH   as PORTC.2      // S2

//
// init_pic() - setup IO pins and other req'd regs
//
sub init_pic()
    TRISA = %00000000       ' all outputs
    TRISB = %00000000       ' all outputs
    TRISC = %00110100       ' D+ and D- are inputs, PB_SWITCH input

  #if (_device = 18F2553)
    ADCON1 = %00001111      ' all digital (disable A/D on all pins)
    CMCON = %00000000       ' disable comparators
    CVRCON = $00            ' and turn off reference
  #endif

    PORTA = $00              ' PORTA outputs Low
    PORTB = $00              ' PORTB outputs Low
    PORTC = $00              ' PORTC outputs Low
end sub
#endif

//
//-------------------------------------------------------------
// LPC USB development board hardware
//-------------------------------------------------------------
//
#if defined(LPC_DEVBD)
dim YELLOW_LED  as PORTC.0      // LED D1
dim RED_LED     as PORTC.1      // LED D2
dim PB_SWITCH   as PORTA.3      // S1 **REQUIRES MCLREN OFF**

//
// init_pic() - setup IO pins and other req'd regs
//
sub init_pic()
    TRISA = %00111111       ' D+ and D- are inputs, all inputs
    TRISB = %01100000       ' RB7/RB4 232 out, RB6/RB5 232 in
    TRISC = %00000000       ' all outs RC0-RC3 leds, RC7-RC4 pickit1 header

    ADCON1 = %0000000       ' Vdd/Vss ref
    ANSEL = 0               ' all digital (disable A/D on all pins)
    ANSELH = 0
    CM1CON0 = %00000000      ' disable comparators
    CM2CON0 = %00000000      ' disable comparators

    PORTA = $00              ' PORTA outputs Low
    PORTB = $00              ' PORTB outputs Low
    PORTC = $00              ' PORTC outputs Low
end sub
#endif

//
//-------------------------------------------------------------
// 46J50 board hardware (USB self-power)
//-------------------------------------------------------------
//
#if defined(EVAL_J50)
dim YELLOW_LED  as PORTC.0      // LED STAT
dim RED_LED     as PORTC.0      // only one led
dim PB_SWITCH   as PORTA.0      // S1

//
// init_pic() - setup IO pins and other req'd regs
//
sub init_pic()
    TRISA = %11111111       ' all inputs
    TRISB = %11111111       ' all in
    TRISC = %11111110       ' all in (D+ and D-), RC0 out
    TRISD = %11111111       ' all in
    TRISE = %11111111       ' all in

    ADCON0 = %00000000      ' Vdd/Vss ref
    ANCON0 = %11111111      ' all digital (disable A/D on all pins)
    ANCON1 = %11111111      ' " , ena bandgap ref
    CM1CON = %00000000      ' disable comparators
    CM2CON = %00000000      ' disable comparators
    'HLVDCON = %10011000     ' ena HLVD in, min voltage

    PORTA = $00              ' PORTA outputs Low
    PORTB = $00              ' PORTB outputs Low
    PORTC = $00              ' PORTC outputs Low
end sub
#endif

//
//-------------------------------------------------------------
// main entry point
//-------------------------------------------------------------
//
main:

init_pic()

delayms(10)

// wait to connect to USB...
high(YELLOW_LED)
repeat
    USBservice()        // if using interrupts, calls to USBService() will be ignored
until (CDC.Attached)
low(YELLOW_LED)

// main program loop
// read bytes and echo them back (use SF SerialCommunicator or another terminal program)
while (true)

    if (CDC.Attached) then
        high(RED_LED)
    else
        low(RED_LED)
    endif

    if (DataAvailable()) then
        high(YELLOW_LED)        // flash the LED to indicate data
        WriteByte(ReadByte())   // read and echo data
        low(YELLOW_LED)
    endif

    // if the USB_SERVICE option has been disabled, you MUST poll the USB connection
    // every 1ms or so to keep it alive and handle pending transactions on the bus...
    USBservice()

end while

